package org.eclipse.smila.sample.pipelet;

import org.apache.axis2.transport.http.HTTPConstants;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.smila.blackboard.BlackboardService;
import org.eclipse.smila.blackboard.path.Path;
import org.eclipse.smila.datamodel.id.Id;
import org.eclipse.smila.datamodel.record.Literal;
import org.eclipse.smila.datamodel.record.RecordFactory;
import org.eclipse.smila.processing.ProcessingException;
import org.eclipse.smila.processing.SimplePipelet;
import org.eclipse.smila.processing.configuration.PipeletConfiguration;

import com.empolis.smila.sample.helloworld.HelloWorldStub;

import demo.hw.server.SayHi;
import demo.hw.server.SayHiDocument;
import demo.hw.server.SayHiResponse;
import demo.hw.server.SayHiResponseDocument;

public class HelloWorldPipelet implements SimplePipelet {

  private final String PROP_IN_ATT_NAME = "IN_ATT_NAME";

  private final String PROP_OUT_ATT_NAME = "OUT_ATT_NAME";

  private String _inAttName;

  private String _outAttName;

  private final Log _log = LogFactory.getLog(HelloWorldPipelet.class);

  public HelloWorldPipelet() {
  }

  public void configure(PipeletConfiguration configuration) throws ProcessingException {
    _inAttName = (String) configuration.getPropertyFirstValueNotNull(PROP_IN_ATT_NAME);
    if (_inAttName.trim().length() == 0) {
      throw new ProcessingException("Property " + PROP_IN_ATT_NAME + " must not be an empty String");
    }

    _outAttName = (String) configuration.getPropertyFirstValueNotNull(PROP_OUT_ATT_NAME);
    if (_outAttName.trim().length() == 0) {
      throw new ProcessingException("Property " + PROP_OUT_ATT_NAME + " must not be an empty String");
    }

  }

  public Id[] process(BlackboardService blackboard, Id[] recordIds) throws ProcessingException {
    for (Id id : recordIds) {
      try {
        String inputValue = "";
        final Path path = new Path(_inAttName);
        if (blackboard.hasAttribute(id, path)) {
          inputValue = blackboard.getLiteral(id, path).getStringValue();
        }
      
        HelloWorldStub ws = new HelloWorldStub("http://localhost:8081/axis2/services/HelloWorld.HelloWorldImplPort");
        ws._getServiceClient().getOptions().setProperty(HTTPConstants.CHUNKED, Boolean.FALSE);
        SayHiDocument sayHiDoc = SayHiDocument.Factory.newInstance();
        SayHi sayHi = sayHiDoc.addNewSayHi();
        sayHi.setArg0(inputValue);
        SayHiResponseDocument respDoc = ws.sayHi(sayHiDoc);
        SayHiResponse response = respDoc.getSayHiResponse();        
        String outputValue = response.getReturn();

        final Literal literal = RecordFactory.DEFAULT_INSTANCE.createLiteral();
        literal.setStringValue(outputValue);
        blackboard.setLiteral(id, new Path(_outAttName), literal);

      } catch (final Exception ex) {
        if (_log.isErrorEnabled()) {
          _log.error("error during execution of HelloWorldPipelet with record " + id, ex);
        }
      }
    } // for

    return recordIds;
  }
}
